shared float4x4 world;
shared float4x4 view;
shared float4x4 projection;
shared float3 cameraPosition;

//light properties
shared float3 lightPosition = float3( -300.00, 300.00, -600.00 );
shared float4 ambientLightColor = float4( 1, 1, 1, 1);
shared float4 diffuseLightColor = float4( 1, 1, 1, 1);
shared float4 specularLightColor = float4( 1, 1, 1, 1);

//material properties
shared float specularPower = 9;     // taille de la tache ( plus c'es petit plus c'est large )
shared float specularIntensity = 0.6;  // intensit de la tache ( 1 signifie tout blanc, 0 pas de reflet )

texture Base;
sampler2D SampBase = sampler_state
{
	Texture = (Base);
	ADDRESSU = WRAP;
	ADDRESSV = WRAP;
	MINFILTER = ANISOTROPIC;
	MAGFILTER = ANISOTROPIC;
	MIPFILTER = ANISOTROPIC;
	MAXANISOTROPY = 8;
};

texture Bump;
sampler2D BumpMap = sampler_state
{
	Texture = (Bump);
	ADDRESSU = WRAP;
	ADDRESSV = WRAP;
	MINFILTER = ANISOTROPIC;
	MAGFILTER = ANISOTROPIC;
	MIPFILTER = ANISOTROPIC;
	MAXANISOTROPY = 8;
};



//////////////////////////////////////////////////////////////
// Example 2.1                                              //
//                                                          //
// This structure will allow the vertex shader to output    //
// normal and position values in TEXCOORD registers.  Like  //
// colors, values in TexCoords are interpolated in screen   //
// space so that each pixel will have properly interpolated //
// normal and world coordinates.  This will enable us to    //
// do the Phong lighting calculation at each pixel for      //
// a much smoother result.                                  //
////////////////////////////////////////////////////////////// 
struct VertexShaderOutputPerPixelDiffuse
{
     float4 Position : POSITION;
     float3 WorldNormal : NORMAL0;
     float3 WorldPosition : TEXCOORD1;
	 float2 Texcoord : TEXCOORD2;
};

struct PixelShaderInputPerPixelDiffuse
{
     float3 WorldNormal : NORMAL0;
     float3 WorldPosition : TEXCOORD1;
	 float2 Texcoord : TEXCOORD2;
};

struct BumpMapVS_OUTPUT
{
	float4 Position : POSITION0;
	float2 Texcoord : TEXCOORD0;
	float3 ViewDirection : TEXCOORD1;
	float3 LightDirection : TEXCOORD2;
	//float3 Normal : NORMAL0;
	//float3 Binormal : BINORMAL0;
	//float3 Tangent : TANGENT0; 
};

struct BumpMapVS_INPUT
{
	float4 Position : POSITION0;
	float2 Texcoord : TEXCOORD0;
	float3 Normal : NORMAL0;
	float3 Binormal : BINORMAL0;
	float3 Tangent : TANGENT0;
};

struct BumpMapPS_INPUT
{
	float4 Position : POSITION0;
	float2 Texcoord : TEXCOORD0;
	float3 ViewDirection : TEXCOORD1;
	float3 LightDirection : TEXCOORD2;
	//float3 Normal : NORMAL0;
	//float3 Binormal : BINORMAL0;
	//float3 Tangent : TANGENT0;
};


//////////////////////////////////////////////////////////////
// Example 2.6                                              //
//                                                          //
// This vertex shader simply sets the output registers so   // 
// that all lighting will be calculated per pixel.          //
//////////////////////////////////////////////////////////////
VertexShaderOutputPerPixelDiffuse PerPixelDiffuseVS(
     float3 position : POSITION,
     float3 normal : NORMAL,
	 float2 tex : TEXCOORD )
{
     VertexShaderOutputPerPixelDiffuse output;

     //generate the world-view-projection matrix
     float4x4 wvp = mul(mul(world, view), projection);
     
     //transform the input position to the output
     output.Position = mul(float4(position, 1.0), wvp);
	 
	 output.Texcoord = tex;
     output.WorldNormal =  mul(normal, world); 
     float4 worldPosition =  mul(float4(position, 1.0), world);
     output.WorldPosition = worldPosition / worldPosition.w;

     //return the output structure
     return output;
}

VertexShaderOutputPerPixelDiffuse BumpMappedVS(
    float3 position : POSITION0,
    float3 normal : NORMAL0,
	float2 tex : TEXCOORD0)
{
     VertexShaderOutputPerPixelDiffuse output;

     //generate the world-view-projection matrix
     float4x4 wvp = mul(mul(world, view), projection);
     
     //transform the input position to the output
     output.Position = mul(float4(position, 1.0), wvp);
	 
	 output.Texcoord = tex;
     output.WorldNormal =  mul(normal, world); 
     float4 worldPosition =  mul(float4(position, 1.0), world);
     output.WorldPosition = worldPosition / worldPosition.w;

     //return the output structure
     return output;
}

//////////////////////////////////////////////////////////////
// Example 2.9                                              //
//                                                          //
// Diffuse + phong in the pixel shader gives the best-      //
// looking results, but is the most GPU intensive of the    //
// techniques in this sample, since more pixels are being   //
// calculated than vertices for our very simple scene.      //
//////////////////////////////////////////////////////////////
float4 BumpMappedPS(PixelShaderInputPerPixelDiffuse input) : COLOR
{
	float3 Normal = normalize(input.WorldNormal+mul(((tex2D( BumpMap, input.Texcoord ).xyz * 2.0f) - 1.0f),world));
     //calculate per-pixel diffuse
    float3 directionToLight = normalize(lightPosition - input.WorldPosition);	
	float diffuseIntensity = saturate( dot(directionToLight, Normal));
    float4 diffuse = diffuseLightColor * diffuseIntensity;

     //calculate Phong components per-pixel
	float3 reflectionVector = normalize(reflect(-directionToLight, Normal));
    float3 directionToCamera = normalize(cameraPosition - input.WorldPosition);
     
     //calculate specular component
    float4 specular = specularLightColor * specularIntensity * 
                      pow(saturate(dot(reflectionVector, directionToCamera)), 
                          specularPower);
						   
	float4 BaseColor = tex2D( SampBase, input.Texcoord );
      
     //all color components are summed in the pixel shader
    float4 color = specular  + diffuse*BaseColor + ambientLightColor*BaseColor;
    color.a = 1.0; 
    return color;
}

float4 DiffuseAndPhongPS(PixelShaderInputPerPixelDiffuse input) : COLOR
{
     //calculate per-pixel diffuse
    float3 directionToLight = normalize(lightPosition - input.WorldPosition);
    float diffuseIntensity = saturate( dot(directionToLight, input.WorldNormal));	
    float4 diffuse = diffuseLightColor * diffuseIntensity;

     //calculate Phong components per-pixel
    float3 reflectionVector = normalize(reflect(-directionToLight, input.WorldNormal));
    float3 directionToCamera = normalize(cameraPosition - input.WorldPosition);
     
     //calculate specular component
    float4 specular = specularLightColor * specularIntensity * 
                      pow(saturate(dot(reflectionVector, directionToCamera)), 
                          specularPower);
						   
	float4 BaseColor = tex2D( SampBase, input.Texcoord );
      
     //all color components are summed in the pixel shader
    float4 color = specular  + diffuse*BaseColor + ambientLightColor*BaseColor;
    color.a = 1.0; 
    return color;
}


BumpMapVS_OUTPUT TestVS( BumpMapVS_INPUT input )
{
	BumpMapVS_OUTPUT output;
	float4x4 wvp = mul(mul(world, view), projection);
	output.Position = mul ( input.Position, wvp );
	output.Texcoord = input.Texcoord;
	
	float3 ObjectPosition = mul( input.Position, world );
	float3 ViewDirection = cameraPosition - ObjectPosition;
	float3 LightDirection = lightPosition - ObjectPosition;
	
	float3 Normal = mul( input.Normal, world );
	float3 Binormal = mul( input.Binormal, world );
	float3 Tangent = mul( input.Tangent, world );
	
	//output.Normal = Normal;
	//output.Binormal = Binormal;
	//output.Tangent = Tangent;
	
	output.ViewDirection.x = dot( Tangent, ViewDirection );
	output.ViewDirection.y = dot( Binormal, ViewDirection );
	output.ViewDirection.z = dot( Normal, ViewDirection );
	
	output.LightDirection.x = dot( Tangent, LightDirection );
	output.LightDirection.y = dot( Binormal, LightDirection );
	output.LightDirection.z = dot( Normal, LightDirection );
	
	return( output );
}

float4 TestPS( BumpMapPS_INPUT input ) : COLOR0
{
	float3 LightDirection = normalize( input.LightDirection );
	float3 Normal = normalize( ( tex2D( BumpMap, input.Texcoord ).xyz * 2.0f ) - 1.0f );
	float NormaldotLight = dot( Normal, LightDirection );
	
	float3 Reflection = normalize( ( ( 2.0f * Normal ) * ( NormaldotLight ) ) - LightDirection );
	float3 ViewDirection = normalize( input.ViewDirection );
	float4 ReflectionDotView = max( 0.0f, dot ( Reflection, ViewDirection ) );
	
	float4 BaseColor = tex2D( SampBase, input.Texcoord );

	float4 TotalAmbient = ambientLightColor * BaseColor;
	float4 TotalDiffuse = diffuseLightColor * NormaldotLight * BaseColor;
	float4 TotalSpecular = specularLightColor * pow( ReflectionDotView, specularPower ) * specularIntensity;

	return( saturate( TotalAmbient + TotalDiffuse + TotalSpecular ) );
	//return( float4( (input.Normal + 1) / 2 , 1 ) ) ;
} 


 technique PerPixelDiffuseAndPhong
{
     
    pass P0
    {
          // set the VertexShader state to the basic
          // vertex shader that will set up inputs for
          // the pixel shader
          VertexShader = compile vs_3_0 PerPixelDiffuseVS();
          
          // set the PixelShader state to the complete Phong Shading
          // implementation of the pixel shader       
          PixelShader = compile ps_3_0 DiffuseAndPhongPS();
    }
}

technique BumpMapping
{
     
    pass P0
    {
          // set the VertexShader state to the basic
          // vertex shader that will set up inputs for
          // the pixel shader
          VertexShader = compile vs_3_0 BumpMappedVS();
          
          // set the PixelShader state to the complete Phong Shading
          // implementation of the pixel shader       
          PixelShader = compile ps_3_0 BumpMappedPS();
    }
}

technique NormalMapping
{
	pass P0
	{
		VertexShader = compile vs_2_0 TestVS();
		PixelShader = compile ps_2_0 TestPS();
	}
} 